# coding:utf8

"""FastWeb is a Python fast-building web frame refered by Tornado,Celery,Thrift"""

# :copyright: (c) 2016-2017 Bslience.  All rights reserved.

from __future__ import absolute_import, division, print_function, with_statement

from fastweb.loader import app
from fastweb.accesspoint import options


__all__ = ['options', 'app']



