# coding:utf8


ERRCODE = {
    'SUC': {'code': 0, 'message': 'success'},
    'ARG': {'code': 1001, 'message': 'invalid arguments'},
    'SVR': {'code': 2001, 'message': 'server error'},
    'TOKEN': {'code': 3995, 'message': 'token invalid'},
    'KEY': {'code': 4001, 'message': 'key not exist'},
    'EXT': {'code': 4002, 'message': 'key exist'},
    'PWD': {'code': 4003, 'message': 'password wrong'},
    'FMT': {'code': 4004, 'message': 'format error'},
    'FT': {'code': 4005, 'message': 'upload file type not support'},
}
